@@ -93,22 +93,76 @@ fn vec_to_array(v: Vec<i8>) -> [i8; 7] {
 
 impl SystemExclusiveData for Common {
     fn from_bytes(data: Vec<u8>) -> Self {
-        eprintln!("SingleCommon");
+        eprintln!("Common data ({} bytes): {:?}", data.len(), data);
 
-        let effects = EffectSettings::from_bytes(data[..31].to_vec());
+        let mut offset = 0;
+        let mut size = 31;
+        let mut start = offset;
+        let mut end = offset + size;
+        let effects_data = data[start..end].to_vec();
+        let effects = EffectSettings::from_bytes(effects_data);
+        offset += size;
 
-        let geq_values = data[31..38].to_vec().iter().map(|n| (n - 64) as i8).collect();  // 58(-6) ~ 70(+6), so 64 is zero
+        size = 7;
+        end = start + size;
+        let geq_data = data[start..end].to_vec();
 
-        let name = String::from_utf8(data[38..46].to_vec()).unwrap();
+        let geq_values = geq_data.iter().map(|n| *n as i8 - 64).collect();  // 58(-6) ~ 70(+6), so 64 is zero
+        offset += size;
+
+        offset += 1;  // skip the drum mark
+        eprintln!("Skipped 'drum mark'");
+
+        size = 8;
+        start = offset;
+        end = offset + size;
+        let name_data = data[start..end].to_vec();
+        let name = String::from_utf8(name_data).unwrap();
         eprintln!("Name = {}", name);
+        offset += size;
 
-        let mutes_byte = data[50];
-        let mut mutes: [bool; 6] = [false; 6];
+        let volume = UnsignedLevel::from(data[offset]);
+        eprintln!("Volume = {}", volume);
+        offset += 1;
+
+        let polyphony = Polyphony::try_from(data[offset]).unwrap();
+        eprintln!("Polyphony = {}", polyphony);
+        offset += 1;
+
+        offset += 1;  // skip the "no use" byte
+        eprintln!("Skipped 'no use'");
+
+        let source_count = data[offset];
+        eprintln!("Sources: {}", source_count);
+        offset += 1;
+
+        let mutes_byte = data[offset];
+        let mut source_mutes: [bool; 6] = [false; 6];
         for i in 0..6 {
-            mutes[i] = mutes_byte.bit(i);
+            source_mutes[i] = mutes_byte.bit(i);
         }
+        offset += 1;
+
+        let amplitude_modulation = AmplitudeModulation::try_from(data[offset]).unwrap();
+        eprintln!("AM = {}", amplitude_modulation);
+        offset += 1;
+
+        size = 6;
+        start = offset;
+        end = start + size;
+        let effect_control_data = data[start..end].to_vec();
+        let effect_control = EffectControl::from_bytes(effect_control_data);
+        eprintln!("Effect control = {:?}", effect_control);
+        offset += size;
+
+        let portamento = if data[offset] == 1 {
+            Portamento::On(UnsignedLevel::from(data[offset + 1]))
+        } else {
+            Portamento::Off
+        };
+        eprintln!("Portamento: {}", portamento);
+        offset += 2;
 
-        let mut offset = 60;
         let mut macro_destinations = Vec::<u8>::new();
         for i in 0..8 {
             eprintln!("Macro {} destination: data = {}", i + 1, data[offset]);
@@ -152,28 +206,27 @@ impl SystemExclusiveData for Common {
             },
         ];
 
+        let switches = SwitchControl {
+            switch1: Switch::try_from(data[offset]).unwrap(),
+            switch2: Switch::try_from(data[offset + 1]).unwrap(),
+            footswitch1: Switch::try_from(data[offset + 2]).unwrap(),
+            footswitch2: Switch::try_from(data[offset + 3]).unwrap(),
+        };
+        eprintln!("Switches: {:?}", switches);
+
         Common {
             effects: effects,
             geq: vec_to_array(geq_values),
             name: name,
-            volume: UnsignedLevel::from(data[46]),
-            polyphony: Polyphony::try_from(data[47]).unwrap(),
-            source_count: data[49],
-            source_mutes: mutes,
-            amplitude_modulation: AmplitudeModulation::try_from(data[51]).unwrap(),
-            effect_control: EffectControl::from_bytes(data[52..58].to_vec()),
-            portamento: if data[58] == 1 {
-                Portamento::On(UnsignedLevel::from(data[59]))
-            } else {
-                Portamento::Off
-            },
+            volume,
+            polyphony,
+            source_count,
+            source_mutes,
+            amplitude_modulation,
+            effect_control,
+            portamento,
             macros: macros,
-            switches: SwitchControl {
-                switch1: Switch::try_from(data[76]).unwrap(),
-                switch2: Switch::try_from(data[77]).unwrap(),
-                footswitch1: Switch::try_from(data[78]).unwrap(),
-                footswitch2: Switch::try_from(data[79]).unwrap(),
-            },
+            switches,
         }
     }
 
@@ -186,7 +239,7 @@ impl SystemExclusiveData for Common {
         result.extend(self.name.clone().into_bytes());  // note clone()
         result.push(self.volume.as_byte());
         result.push(self.polyphony as u8);
-        result.push(0);
+        result.push(0);  // "no use"
         result.push(self.source_count);
 
         let mut mute_byte = 0x00;
@@ -335,18 +388,36 @@ impl Default for SinglePatch {
 
 impl SystemExclusiveData for SinglePatch {
     fn from_bytes(data: Vec<u8>) -> Self {
-        let mut offset = 0;
+        let mut offset: usize = 0;
+        let mut start: usize = 0;
+        let mut end: usize = 0;
+        let mut size: usize = 0;
 
-        let original_checksum = data[0];
+        /*
+        let original_checksum = data[offset];
         eprintln!("original checksum = {:#02x}", original_checksum);
+        offset += 1;
+        */
 
-        let common = Common::from_bytes(data[1..82].to_vec());
-        offset += 81;
+        size = 81;
+        end = start + size;
+        let common_data = data[start..end].to_vec();
+        eprintln!("Starting to parse source common...");
+        let common = Common::from_bytes(common_data);
+        offset += size;
+
+        eprintln!("Starting to parse {} sources, offset = {}", common.source_count, offset);
+
+        size = 86;
         let mut sources = Vec::<Source>::new();
         for i in 0..common.source_count {
-            let source = Source::from_bytes(data[offset..offset + 86].to_vec());
+            start = offset;
+            end = start + size;
+            let source_data = data[start..end].to_vec();
+            eprintln!("Parsing source {}...", i + 1);
+            let source = Source::from_bytes(source_data);
             sources.push(source);
-            offset += 86;
+            offset += size;
         }
 
         let mut additive_kits = BTreeMap::<String, AdditiveKit>::new();
@@ -354,9 +425,13 @@ impl SystemExclusiveData for SinglePatch {
         // How many additive kits should we expect then?
         let kit_count = sources.iter().filter(|s| s.oscillator.wave.is_additive()).count();
         let mut kit_index = 0;
+        size = 806;
         while kit_index < kit_count {
-            let kit = AdditiveKit::from_bytes(data[offset..offset + 806].to_vec());
-            offset += 806;
+            start = offset;
+            end = start + size;
+            let kit_data = data[start..end].to_vec();
+            let kit = AdditiveKit::from_bytes(kit_data);
+            offset += size;
             let kit_name = format!("s{}", kit_index + 1);
             additive_kits.insert(kit_name, kit);
             kit_index += 1;
@@ -414,3 +489,66 @@ impl fmt::Display for SinglePatch {
         write!(f, "{}\nSOURCES:\n{}", self.common, sources_str)
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::{*};
+
+    #[test]
+    fn test_common_from_bytes() {
+        let data = vec![
+            // Effect data
+            0x00,  // effect algorithm
+            0x00,  // reverb type
+            0x02,  // reverb dry/wet
+            0x02,  // reverb param 1
+            0x0d,  // reverb param 2
+            0x41,  // reverb param 3
+            0x0a,  // reverb param 4
+            0x10,  // effect 1 type
+            0x00,  // effect 1 depth
+            0x58,  // effect 1 param 1
+            0x33,  // effect 1 param 2
+            0x69,  // effect 1 param 3
+            0x22,  // effect 1 param 4
+            0x1d, 0x00, 0x4a, 0x00, 0x00, 0x00,  // effect 2 (as above)
+            0x24, 0x00, 0x04, 0x3a, 0x04, 0x38,  // effect 3 (as above)
+            0x2a, 0x00, 0x0c, 0x0c, 0x63, 0x00,  // effect 4 (as above)
+            0x42, 0x41, 0x40, 0x40, 0x3f, 0x3e, 0x41, // GEQ
+            0x00,  // "drum mark"
+            0x57, 0x69, 0x7a, 0x6f, 0x6f, 0x49, 0x6e, 0x69,  // name "WizooIni"
+            0x73,  // volume
+            0x00,  // polyphony
+            0x00,  // "no use"
+            0x02,  // no. of sources
+            0x01,  // source mutes
+            0x00,  // AM
+
+            // Effect control
+            0x02, 0x01, 0x40,  // control source 1, destination, depth
+            0x01, 0x03, 0x40,  // control source 2, destination, depth
+            0x00, 0x00,  // portamento flag and p. speed
+
+            // Macro controllers
+            0x00, 0x00,  // macro controller 1 params 1 and 2
+            0x00, 0x00,  // macro controller 2 params 1 and 2
+            0x00, 0x00,  // macro controller 3 params 1 and 2
+            0x00, 0x00,  // macro controller 4 params 1 and 2
+            0x40, 0x40,  // macro controller 1 param 1 and param 2 depths
+            0x40, 0x40,  // macro controller 2 param 1 and param 2 depths
+            0x40, 0x40,  // macro controller 3 param 1 and param 2 depths
+            0x40, 0x40,  // macro controller 4 param 1 and param 2 depths
+            0x00, 0x00, 0x00, 0x00,  // SW1, SW2, F.SW1, F.SW2
+        ];
+
+        let common = Common::from_bytes(data);
+        assert_eq!(common.name, "WizooIni");
+    }
+
+    #[test]
+    fn test_single_patch_from_bytes() {
+        let data: [u8; 1070] = include!("WizooIni.in");
+        let single_patch = SinglePatch::from_bytes(data[10..].to_vec());  // skip sysex header and checksum
+        assert_eq!(single_patch.common.name, "WizooIni");
+    }
+}
